/**********************************************************************
 * client_data.h                                                      *
 *                                                                    *
 * Data Structures for the Client                                     *
 *                                                                    *
 * Brian Hoffman                                                      *
 * bhoffman@alum.mit.edu                                              *
 * ICFP Programming Contest 2002                                      *
 *                                                                    *
 **********************************************************************/

#ifndef CLIENT_DATA_H
#define CLIENT_DATA_H

#include "client_defs.h"

/*******************************
 * Robot Structures            *
 *******************************/
typedef struct _robot_entry
{
  int x;
  int y;

  struct _robot_entry *next;
  struct _robot_entry *prev;

} Robot_entry;

/******************************
 * Package Structures         *
 ******************************/
typedef struct _package_entry
{
  int id;
  int dest_x;
  int dest_y;
  int weight;

  struct _package_entry *next;
  struct _package_entry *prev;

} Package_entry;

/*******************************
 * Map Data Structures         *
 *******************************/
typedef struct _map_node
{
  int  x;         /* Node X Location          */
  int  y;         /* Node Y Location          */
  char type;      /* Type of terrain          */
  char visited;   /* Have I been here before? */
  char packages;  /* Number of packages seen at this node */

  /* Data for Path Planner
   * links : Set a bit for each direction the robot can move.
   *
   * 1 = Can Move That Direction, 0 = Can't Move.
   * N = 0x1, S = 0x2, E = 0x4, W = 0x8
   */
  char links;
  int  g;
  int  h;
  int  f;
  struct _map_node *parent;

} Map_node;

/*******************************
 * Path Finding Structures     *
 *******************************/
/* These look the same, but name them differently
 * so type checking works in our favor.
 */
typedef struct _pf_pque_entry
{
  struct _map_node *node;

  struct _pf_pque_entry *prev;
  struct _pf_pque_entry *next;

} Pf_pque_entry;

typedef Pf_pque_entry Pf_list_entry;
typedef Pf_pque_entry Path_entry;

/*******************************
 * Global State Data           *
 *******************************/
typedef struct _game_state
{
  int socket_open; /* Is the socket open? */
  int socket;      /* Socket handle       */

  int map_size_x;  /* Current Map Width   */
  int map_size_y;  /* Current Map Height  */

  int robot_id;    /* Our ID Number             */
  int max_weight;  /* Our Max Carrying Capacity */
  int money;       /* Our current money         */

  int robot_x;     /* Our current X position    */
  int robot_y;     /* Our current Y position    */
  int weight;      /* Our current Weight        */
  int alive;       /* Am I Alive?               */

  int replan_path; /* We've been pushed, or otherwise
                    * gotten off the path, and need to
                    * replan it.
                    */

  int score_guess;

  Robot_entry   *robot_list;    /* List of Robots   */
  Package_entry *pkg_unpicked;  /* List of Packages */
  Package_entry *pkg_picked;    /* List of Packages */
  Path_entry    *active_path;   /* Our Current Path */

} Game_state;

#endif
